###### Function aganovas Several anovas and Kruskal Wallis analyses ############
### Created: 180621 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### V0.0.1: Not working yet.
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#### Roxigen documentation ####
#' Anovas and Kruskal Wallis
#'
#' Aggregates into the same list the results of several parametric (anovas)
#' and non-parametric (Kruskal-Wallis) analyses for several independent factors
#' and dependent numeric variables.
#'
#' Each analysis uses \code{\link{agaovhsd}} for anovas and
#' \code{\link{agkruskalfigs}} for kruskal wallis analyses.
#'
#' @param df0 A data frame
#' @param x Numbers or character vector pointing to the x variable inside "df0"
#'   dataframe. Should be a factor variable.
#' @param y Numbers or character vector pointing to the y variable inside "df0"
#'   dataframe. Should be a numeric variable.
#' @param ... Names needed for next parameters.
#' @param res List of results to add the results to. Default to NULL (new list).
#' @param lettersheightbarplot Changes the letters height in the barplots.
#' Default NULL, lettersheightbarplot = mean(df0[[y]]) / 4
#' @param lettersheightviolin Changes the letters height in the violin plots.
#' Default NULL,
#' lettersheightviolin = max(df0[[y]]) + ((max(df0[[y]]) - min(df0[[y]])) / 4)
#' @param alpha0 Significance level for post hoc letters.
#' @param aov If ANOVA analyses are wanted. Default TRUE.
#' @param kw If Kruskal Wallis analyses are wanted. Default TRUE.
#' @param maxlevels Maximum number of levels for categorical vars to be
#'        considered. It should be 9 or less. If more, the densities
#'        plots will not have enough colors.
#' @param barplot If barplot is wanted in the output. Default TRUE.
#' @param violin If violin plot is wanted in the output. Default TRUE.
#' @param histog If histograms plot is wanted in the output. Default TRUE.
#' @param xlabs Changes the x axis labels in plots.
#' @param ylabs Changes the y axis labels in plots.
#' @param xangle Changes the angle of text for x axis label in plots.
#' @param xhjust Changes the horizontal justification of text for x axis label
#' in plots.
#' @param ggtheme Theme for ggplot barplot. Default = ggplot2::theme_bw().
#'
#' @return
#' A list with sublists for each dependent variable and sublists for  plots,
#' plot captions (capplots), tables and tables captions (captables).
#'
#' For each independent and dependent variable, the format will be:
#'
#' - $Aov_dependent$independent$plots$plotname
#' - $Aov_dependent$independent$capplots$plotname
#' - $Aov_dependent$independent$tables$tablename
#' - $Aov_dependent$independent$captables$tablename
#'
#'If a results list is pointed in parameters,
#' it will add the results to this list.
#'
#' @export
#'
#' @examples
#' A <- aganovas(df0 = iris, x = 5, y = 3:4)
#' A
#'
#' df0 <- iris
#' df0$fakevar <- rep(c(TRUE, TRUE, FALSE))
#' A <- aganovas(df0 = iris, x = 5:6, y = 3:4)
#' A
#'
#'
#' @seealso
#' \code{\link{agaovhsd}} for details on the analyses of variance (anova) and
#' \code{\link{agkruskalfigs}} for details on the non-parametric Kruskal-Wallis
#' analyses. Also \code{\link{agTanovas}} and \code{\link{agTkruskal}} are used
#' for the summary tables.
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
##### Init function ####
# df0 <- iris
# df0$categ <- rep(c(TRUE, TRUE, FALSE))
# x <- 5:6
# y <- 2:4
# res = res
# lettersheightbarplot = NULL
# lettersheightviolin = NULL
# alpha0 = 0.05
# aov = TRUE
# kw = TRUE
# barplot = TRUE
# violin = TRUE
# histog = TRUE
# xlabs = NULL
# ylabs = NULL
# xangle = NULL
# xhjust = NULL
# ggtheme = ggplot2::theme_bw()

#### Function ####
aganovas <- function(df0, x, y, ..., res = NULL,
                     lettersheightbarplot = NULL,
                     lettersheightviolin = NULL,
                     alpha0 = 0.05, ggtheme = ggplot2::theme_bw(),
                     aov = TRUE, kw = TRUE, maxlevels = 9,
                     barplot = TRUE, violin = TRUE, histog = TRUE,
                     xlabs = NULL, ylabs = NULL, xangle = NULL, xhjust = NULL
) {
  #### Require ####
  # agrequire(c("plyr", "car", "agricolae", "ggplot2"))
  #
  #### Check ####
  if (TRUE) {
    #### df0 ####
    df0 <- agcheckdf(df0, dfname = "df0", funame = "aganovas")

    #### alpha0 ####
    if (!is.numeric(alpha0)) stop("aganovas(): 'alpha0' must be numeric.")
    if (alpha0 <= 0 || alpha0 >= 1) stop(paste("aganovas(): 'alpha0' must be",
                                               "a number between 0 and 1."))

    #### Variables assignations ####
    x <- agcheckvar(df0 = df0, v = x, funame = "aganovas", vname = "x",
                    dfname = "df0")
    y <- agcheckvar(df0 = df0, v = y, funame = "aganovas", vname = "y",
                    dfname = "df0")

    #### Factor variables x ####
    df0 <- agcheckfactor(df0, x, maxlevels = maxlevels, funame = "aganovas",
                         dfname = "df0", vname = "x")

    #### Numeric variables y ####
    ### Coerce
    df1 <- as.data.frame(lapply(df0, as.numeric))

    ### check
    for (i in y) {
      if (!is.numeric(df1[[i]])) {
        nam <- names(df1[i])
        stop(paste0("aganovas(): 'y' variables must be numeric. '",
                    nam, "' is not numeric."))
      } # if not numeric
    } # for i in y if not numeric, stop.

    #### Letters height ####
    if (!is.null(lettersheightbarplot)) {
      if (!any(length(lettersheightbarplot) == 1,
               length(lettersheightbarplot) == length(y))) {
        stop(paste("aganovas(): lettersheightbarplot must be either 1 or",
                   "the same length than dependent variables (y)"))
      } # If not length 1 or the same than y, error.
    } # if not NULL

    if (!is.null(lettersheightviolin)) {
      if (!any(length(lettersheightviolin) == 1,
               length(lettersheightviolin) == length(y))) {
        stop(paste("aganovas(): lettersheightviolin must be either 1 or",
                   "the same length than dependent variables (y)"))
      } # If not length 1 or the same than y, error.
    } # if not NULL

  } # All checks

  #### Activate results list ####
  if (is.null(res)) res <- list()

  #### Analyses for each dependent "y" and each dependent x ####
  # i = y[1]
  # j = x[1]
  k = 0
  for (i in y) {
    k <- k + 1 # counts dependent variable
    for (j in x) {
      #### If is the same, next ####
      if (i == j) next

      #### Letters heigths ####
      if (is.null(lettersheightbarplot)) {
        lettersheightbarplot0 <- mean(df0[[i]]) / 4
      } else {
        if (length(lettersheightbarplot) > 1)
          lettersheightbarplot0 <- lettersheightbarplot[k]
        if (length(lettersheightbarplot) == 1)
          lettersheightbarplot0 <- lettersheightbarplot
      } # valor de lettersheightbarplot0

      if (is.null(lettersheightviolin)) {
        lettersheightviolin0 <- max(df0[[i]]) +
          ((max(df0[[i]]) - min(df0[[i]])) / 4)
      } else {
        if (length(lettersheightviolin) > 1)
          lettersheightviolin0 <- lettersheightviolin[k]
        if (length(lettersheightviolin) == 1)
          lettersheightviolin0 <- lettersheightviolin
      } # valor de lettersheightviolin0

      #### ANOVA ####

      a <- agaovhsd(df0 = df0, x = j, y = i, dep = names(df0)[i],
                    barplot = barplot, violin = violin,
                    xlab0 = xlabs[j], ylab0 = ylabs[i],
                    xangle = xangle, xhjust = xhjust,
                    lettersheightbarplot = lettersheightbarplot0,
                    lettersheightviolin = lettersheightviolin0,
                    ggtheme = ggtheme, alpha0 = alpha0)

      #### Kruskal Wallis ####
      w <- agkruskalfigs(df0 = df0, x = j, y = i, xlab = xlabs[j],
                         lettersheight = lettersheightbarplot0)

      #### Add to list ####
      yn <- names(df0[i])
      xn <- names(df0[j])

      den <- paste0("Aov_", yn)
      inn <- xn

      res[[den]][[inn]]$plots$violin <- a$plots$violin
      res[[den]][[inn]]$capplots$violin <- a$capplots$violin

      res[[den]][[inn]]$plots$Densities <- w$plots$Densities
      res[[den]][[inn]]$capplots$Densities <- w$capplots$Densities

      res[[den]][[inn]]$plots$anovabp <- a$plots$barplot
      res[[den]][[inn]]$capplots$anovabp <- a$capplots$barplot

      res[[den]][[inn]]$plots$kruskalbp <- w$plots$BarPlot
      res[[den]][[inn]]$capplots$kruskalbp <- w$capplots$BarPlot

      res[[den]][[inn]]$tables$anovaA <- a$tables$TA
      res[[den]][[inn]]$captables$anovaA <- a$captkruskalables$TA
      res[[den]][[inn]]$tables$anovaB <- a$tables$TB

      res[[den]][[inn]]$tables$kw <- w$tables$kw
      res[[den]][[inn]]$captables$kw <- w$captables$kw

      res[[den]][[inn]]$title <- paste(yn, "vs.", xn)
      res[[den]][[inn]]$subtitle <- paste0('Effect of ',xn, ' on ', yn, ".")
    } #  j for each x variable

    #### Title ####
    xl <- length(x)
    xn <- agvectorastext(names(df0)[x])
    res[[den]]$title <- paste0("Anovas and Kruskal-Wallis for '",  yn, "'")
    res[[den]]$subtitle <- paste0("Effect of ", xl, " variables on '",  yn, "': ",
                                  xn, ". Use anova results if parametric and ",
                                  "Kruskal-Wallis if not parametric.")
  } # i for each y variable
  #### Output ####
  return(res)
}


agvectorastext <- function(vector, sep = ", ") {
  vector <- as.character(vector)
  txt <- vector[1]
  if (length(vector) > 1) {
    for (i in 2:length(vector)) {
      txt <- paste(txt, vector[i], sep = sep)
    }
  }
  return(txt)
}

