#### Function agaovhsd Post Hoc anova with Levene, shapiro, plot ###############
### Created: 170526 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### v0.0.5 180512: Add roxygen2 documentation and separate needed functions
### v0.0.4 180429: Install required packages without activ: plyr, car, ggplot
### V0.0.3 180416: Fixed bug with NA (na.exclude)
### V0.0.2 171012: Fixed problems
### V0.0.1: Does Normality of residuals test.
###         H0= Normality.
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#### Roxigen ####
#' Anova with tuckey post hoc HSD
#'
#' Makes analysis of variance and Tuckey Honest Significant Differences Post Hoc.
#' Makes tables and plots to ilustrate the analysis.
#'
#' @param df0 The data frame where x and y are.
#' @param x String or number pointing to the independent variable inside "df0"
#'  dataframe. Should be a factor variable.
#' @param y SString or number pointing to the dependent variable inside "df0"
#'  dataframe. Should be a numeric variable.
#' @param ... names needed for next parameters.
#' @param barplot If barplot is wanted in the output.
#' @param violin If violin plot is wanted in the output.
#' @param xlab0 x variable name for plots and tables. If NULL, the actual
#' variable name will be used.
#' @param ylab0 y variable name for plots and tables. If NULL, the actual
#' variable name will be used.
#' @param xangle Changes the angle of text for x axis label in plots.
#' @param xhjust Changes the horizontal justification of text for x axis label
#' in plots.
#' @param lettersheightbarplot Changes the letters height in the barplot.
#' Default = mean(df0[[y]]) / 4
#' @param lettersheightviolin Changes the letters height in the violin plot.
#' Default = max(df0[, y]) + ((max(df0[, y]) - min(df0[, y])) / 4)
#' @param ggtheme Theme for ggplot barplot. Default = ggplot2::theme_bw().
#' @param alpha0 Significance level for post hoc letters.
#' @param maxlevels Maximum number of levels for categorical vars to be
#'        considered. It should be 9 or less. If more, the densities
#'        plots will not have enough colors.
#'
#' @import agricolae
#' @import ggplot2
#' @import plyr
#' @import car
#'
#' @return
#' Returns a list with two tables (principal statistics and principal anova statistics) two plots (barplot and violin plot), Tuckey hsd distance and groups and residuals:
#'
#'  - $plots$barplot The barplot with posthoc letters and error bars.
#'
#'  - $plots$violin  The violin plot with boxplot inside and posthoc letters.
#'
#'  - $capplots$barplot Proposed caption for the barplot.
#'
#'  - $capplots$violin Proposed caption for the violin plot.
#'
#'  * tables$TA Table of principal statistics: levels, mean, standard deviation,
#'        , standard error, skewness, kurtosis and shapiro.test for normality.
#'
#'  - tables$TB Table of anova statistics: df (degrees of freedom), sum sq,
#'        Mean Sq, F value, P, HSD, eta.sq, Levene  H0= Variances are equal.
#'        Requires library(cars) and Shapiro-Wilk test for residuals.
#'
#'  - captables$TA Caption for table A and B. Intended to be together.
#'
#'  - $hsd Tukey HSD PostHoc test value
#'
#'  - $groups Groups for Tukey HSD
#'
#'  - $residuals Residuals from anova. For ulterior analyses or histograms.
#'
#' @details
#'
#' Require plyr and car to be installed (not necessarily activated).
#'
#' Require agricolae and ggplot2 active.
#'
#' @seealso
#' The packages plyr, car, agricolae, ggplot are used. Also functions
#' \code{\link{agbarplot}} and \code{\link{agviolin}}.
#'
#' \code{\link[car]{leveneTest}}, for Levene test details.
#'
#' \code{\link[ggpubr]{ggdensity}}, for density plot details.
#'
#' \code{\link[agricolae]{skewness}}, for skewness measurement details.
#'
#' \code{\link[agricolae]{kurtosis}}, for kurtosis measurement details.
#'
#' \code{\link[stats]{shapiro.test}}, for Shapiro-Wilk normality test details.
#'
#' \code{\link[agricolae]{HSD.test}}, for post-hoc test details.
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#' @examples
#' res <- list()
#' aovhsd <- agaovhsd(df0 = iris, x = "Species", y = "Petal.Length",
#'                    xlab0 = "Iris species")
#' (res$iris$plots$petlength$barplot <- aovhsd$plots$barplot)
#' (res$iris$plots$petlength$violin <- aovhsd$plots$violin)
#' (res$iris$capplots$petlength$barplot <- aovhsd$capplots$barplot)
#' (res$iris$capplots$petlength$violin <- aovhsd$capplots$violin)
#' (res$iris$tables$petlength$TA <- aovhsd$tables$TA)
#' (res$iris$tables$petlength$TB <- aovhsd$tables$TB)
#' (res$iris$captables$petlength$TA <- aovhsd$captables$TA)
#'
#' ### Tables should be something like this:
#' # $TA
#' # Iris species  N  mean HSD        sd         se       skew   kurtosis    Shapiro
#' # 1       setosa 50 1.462   c 0.1736640 0.02455980  0.1063939  1.0215761 0.05481147
#' # 2   versicolor 50 4.260   b 0.4699110 0.06645545 -0.6065077  0.0479033 0.15847784
#' # 3    virginica 50 5.552   a 0.5518947 0.07804970  0.5494446 -0.1537786 0.10977537
#' #
#' # $TB
#' # Analysis of Variance Table
#' #
#' # Response: Petal.Length
#' # Df Sum Sq Mean Sq F value     Pr(>F)     HSD  eta.sq     Levene  Shapiro
#' # Species     2 437.10 218.551  1180.2 2.8568e-91 0.20378 0.94137 1.2161e-08
#' # Residuals 147  27.22   0.185                                               0.036764
#' #
#' # $hsd
#' # [1] 0.20378
#' #
#' # $groups
#' # Petal.Length HSD Iris species
#' # virginica         5.552   a    virginica
#' # versicolor        4.260   b   versicolor
#' # setosa            1.462   c       setosa
#' #
#' # $residuals
#'
#' @export
#'
#### Initialize ####
# df0 <- iris
# x <- 2
# y <- 5
# barplot = TRUE
# violin = TRUE
# xlab0 <- "Iris species"
# barplot = TRUE
# xlab0 = NULL
# ylab0 = NULL
# xangle = NULL
# xhjust = NULL
# lettersheightbarplot = mean(df0[[y]]) / 4
# lettersheightviolin = max(df0[, y]) + ((max(df0[, y]) - min(df0[, y])) / 4)
# ggtheme = ggplot2::theme_bw()
# alpha0 = 0.05

##### FUNCTION ####
agaovhsd <- function(df0, x, y, ..., barplot = TRUE, violin = TRUE,
                     xlab0 = NULL, ylab0 = NULL, xangle = NULL, xhjust = NULL,
                     lettersheightbarplot = mean(df0[[y]]) / 4,
                     lettersheightviolin = max(df0[, y]) +
                       ((max(df0[, y]) - min(df0[, y])) / 4),
                     ggtheme = ggplot2::theme_bw(), alpha0 = 0.05,
                     maxlevels = 9){
  #### Require ####
  # agrequire(c("agricolae", "plyr", "car", "ggplot2"))

  #### Check ####
  if (TRUE) {
    #### df0 ####
    df0 <- agcheckdf(df0, dfname = "df0", funame = "agaovhsd")

    #### alpha0 ####
    if (!is.numeric(alpha0)) stop("agaovhsd(): 'alpha0' must be numeric.")
    if (alpha0 <= 0 || alpha0 >= 1) stop(paste("agaovhsd(): 'alpha0' must be",
                                               "a number between 0 and 1."))

    #### Variables assignations ####
    x <- agcheckvar(df0 = df0, v = x, funame = "agaovhsd", vname = "x",
                    dfname = "df0")
    y <- agcheckvar(df0 = df0, v = y, funame = "agaovhsd", vname = "y",
                    dfname = "df0")

    #### Only one variable x and y ####
    if (length(df0[x]) != 1) stop("agaovhsd(): 'x' must be ONE variable.")
    if (length(df0[y]) != 1) stop("agaovhsd(): 'y' must be ONE variable.")

    #### Factor variable x ####
    df0 <- agcheckfactor(df0, x, maxlevels = maxlevels, funame = "agaovhsd",
                         dfname = "df0", vname = "x")

    #### Numeric variables y ####
    ### Coerce
    df1 <- as.data.frame(lapply(df0, as.numeric))

    ### check
    for (i in y) {
      if (!is.numeric(df1[[i]])) {
        nam <- names(df1[i])
        stop(paste0("agaovhsd(): 'y' variables must be numeric. '",
                    nam, "' is not numeric."))
      } # if not numeric
    } # for i in y if not numeric, stop.
  } # All checks


  #### EXTRACT DATA FOR ANALYSIS ####
  xn <- names(df0)[x]
  yn <- names(df0)[y]
  if (is.null(xlab0)) xlab0 <- xn
  if (is.null(ylab0)) ylab0 <- yn

  df1 <- na.exclude(df0[, c(x, y)])
  names(df1) <- c("x", "y")

  #### Init table of means A ####
  ll <- length(levels(df1$x))
  ta <- data.frame(x = levels(df1$x),
                     N = numeric(ll),
                     mean = numeric(ll),
                     sd = numeric(ll),
                     se = numeric(ll),
                     skew = numeric(ll),
                     kurtosis = numeric(ll),
                     Shapiro = numeric(ll)
  )
  if (is.null(xlab0))  names(ta)[1] <- xn
  if (!is.null(xlab0)) names(ta)[1] <- xlab0

  #### Table of means ####
  df1$x <- as.factor(df1$x)
  # i = 1
  for (i in 1:ll) {
    s <- subset(df1, df1$x == levels(df1$x)[i])
    ta$N[i] <- length(s$y)
    ta$mean[i] <- mean(s$y, na.rm = TRUE)
    ta$sd[i] <- sd(s$y)
    ta$se[i] <- sd(s$y)/sqrt(length(s$y))
    ta$skew[i] <- agricolae::skewness(s$y)
    ta$kurtosis[i] <- agricolae::kurtosis(s$y)
    ta$Shapiro[i] <- ifelse(is.nan(agricolae::skewness(s$y)), NA,
                            shapiro.test(s$y)$p.value)
  }

  #### Anova table ####
  frm <- paste(yn, xn, sep = "~")
  AOV <- aov(formula(frm), data = df0)
  T1b <- anova(AOV)
# TukeyHSD(AOV)

  #### HSD test ####
  if (!is.na(T1b$`Pr(>F)`[1])) {   # If there are any differences (not all equal)
    hsdt <- agricolae::HSD.test(AOV, xn, group = TRUE, DFerror = df.residual(AOV),
                     MSerror = deviance(AOV)/DFerror, alpha = alpha0)
    groups <- hsdt$groups
    #    names(groups)[c(1,3)] <- c(xlab0, "HSD")   # Eliminado tras actualizacion 170922 (groups ha eliminado columna de nombres tratamientos)
    names(groups)[2] <- "HSD"                           # Añadido tras actualizacion 170922
    groups$rownames <- rownames(groups)                 # Añadido tras actualizacion 170922

    ### Add HSD to table b
    # hsd <- hsdt$statistics$HSD
    hsd <- hsdt$statistics$MSD
    T1b$HSD <- c(hsd, "")


    W <- FALSE
  } else {
    W <- TRUE
    warning(paste0("It is not possible to calculate a p for different '", xlab0,
                   "' in agaovhsd(). No differences between groups. ",
                   "Also Shapiro-Wilk test not possible."))
    ### fake table
    groups <- data.frame(notnecessary = NA,
                         HSD = rep("a", length(levels(df1[[1]]))),
                         rownames = levels(df1[[1]])
                         )
  }
  names(groups)[names(groups) == "rownames"] <- xn     # Añadido tras actualizacion 170922
  ### Order groups
  #    groups <- groups[order(groups[, xlab0]), ]       # No es necesario

  ### Add letters to table
  ta <- plyr::join(ta, groups[c(2, 3)], by = xlab0)      # modificado tras actualizacion 170922
  ta[2:8] <- as.data.frame(lapply(ta[2:8], as.numeric))

  ta <- ta[c(1:3,9,4:8)]


  ####  Effect size for ANOVA ####
  aov.table = summary(AOV)
  ss = aov.table[[1]]$"Sum Sq"
  eta.sq = ss[1] / (ss[1] + ss[2])
  T1b$eta.sq <- c(eta.sq, NA)

  ####   Levene's test ####
  T1b$Levene <- car::leveneTest(formula(frm), data = df0, center = "mean")$'Pr(>F)'

  ####  Shapiro test (residuals) ####
  if (!W) T1b$Shapiro <- c(NA, shapiro.test(AOV$residuals)$p.value)

  #### captables ####
  # xn <- which(x %in% names(df0))
  # yn <- which(y %in% names(df0))
  # if (is.null(xlab0)) xlab0 <- names(df0)[xn]
  # if (is.null(ylab0)) ylab0 <- names(df0)[yn]

  captab <- paste0("Anova results for the effect of '", xlab0, "' on '", ylab0,
                   "'. Different letters represent significant differences",
                   " for alpha = ", alpha0,  ".")

  #### Init Return ####
    LS <- list()

  #### Barplot ####
  if (barplot) {
    BP <- agbarplot(df0 = df0, x = x, y = y, xlab0 = xlab0, ylab0 = ylab0,
            letters0 = ta$HSD, lettersheight = lettersheightbarplot,
            ggtheme = ggtheme, xangle = xangle, hjust = xhjust)

  LS$plots$barplot <- BP$barplot
  LS$capplots$barplot <-
    paste0(BP$caption, " Different letters represent significant differences",
           " for alpha = ", alpha0,  ".")
  }

  #### Violin plot ####
  if (violin) {
    xn <- which(names(df0) %in% x)
    yn <- which(names(df0) %in% y)
    vp <- agviolin(x = df0[x], y = df0[y], xlab0 = xlab0, ylab0 = ylab0,
                   letters0 = ta$HSD, lettersheight = lettersheightviolin,
                   ggtheme = ggtheme, xangle = xangle, hjust = xhjust)

    LS$plots$violin <- vp$plots
    LS$capplots$violin <-
      paste0(vp$capplots,
             ". Different letters represent significant differences",
             " for alpha = ", alpha0,  ".")
  }

  #### RETURN ####
  LS$tables$TA <- ta
  LS$tables$TB <- T1b
  LS$captables$TA <- captab
  LS$hsd <- hsd
  LS$groups <- groups
  LS$residuals <- AOV$residuals

  return(LS)
}

