####### Function agbarplotinteraction Barplot for interaction ##################
### Created: 180428 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### v0.0.3 180512: Add roxygen2 documentation.
### V0.0.2 180429: Working version with sizes and colours.
### V0.0.1: Still not Working version
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#### Roxigen ####
#' Barplot for interaction.
#'
#' Does the barplot for interaction with a covariate factor.
#'
#' @param df0 A data frame
#' @param x String or number pointing to the x variable inside "df0"
#'   dataframe. Should be a factor.
#' @param y String or number pointing to the y variable inside "df0"
#'   dataframe. Should be numeric.
#' @param cov String or number pointing to the covariate inside "df0"
#'   dataframe. Should be a factor.
#' @param ... Names needed for next parameters.
#' @param xlab0 Changes the x axis label in barplot.
#' @param ylab0 Changes the y axis label in barplot.
#' @param covlab Changes the covariate label in barplot.
#' @param letters0 Post Hoc letters to include into the barplot for main effect.
#'   If null not added.
#' @param letterscov Post Hoc letters to include into the barplot for covariate
#'   effect. It is the effect into each x level. If null not added.
#' @param lettersheight0 Changes the letters0 height in the barplot.
#'   Default = max(df2$Mean) + max(df2$Mean) / 5
#' @param lettersheightcov Changes the letterscov height in the barplot.
#'   Default = mean(na.exclude(df0[[y]])) / 4
#' @param letters0size Changes the size of the letters0. Default = 6.
#' @param letterscovsize Changes the size of the letterscov. Default = 5.
#' @param letters0col Changes the colour of the letters0. Default = "darkgrey".
#' @param letterscovcol Changes the colour of the letterscov. Default = "black".
#' @param ggtheme Changes theme for barplot. Default = ggplot2::theme_bw().
#' @param width Bar width. By default, set to 90% of the resolution of the data.
#'   See geom_bar() help.
#' @param dodge Dodging width, when different to the width of the individual
#'   elements. This is useful when you want to align narrow geoms with wider
#'   geoms. See position_dodge(width) help.
#'
#' @return
#' A ggplot object with a barplot with covariate bars for each x variable.
#'
#' @import ggplot2
#'
#' @export
#'
#' @examples
#' df0 <- iris
#' df0$cov <- rep(c("A", "B", "C"), 50)
#' agbarplotinteraction(df0, x = 5, y = 1, cov = 6, covlab = "FakeGroup",
#'                      letters0 = c("a", "b", "c"),
#'                      letterscov = letters[1:9], letters0col = "red")
#'
#' @details
#' Require ggplot2.
#'
#' @seealso
#' \code{\link{agautoanalyses::agrequire}},
#' \code{\link{ggplot2::ggplot}}
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#### Init ####
# df0 <- iris
# df0$cov <- rep(c("a", "b", "c"), 50)
# x <- 5
# y <- 1
# cov <- 6
# xlab0 = NULL
# ylab0 <- "Sepal width (cm)"
# covlab <- "Covariate ordered"
# letters0 <- c("a", "b", "c")
# letterscov <- rep(c("a", "b", "c"), 3)
# lettersheight0 = NULL
# letters0size = 6
# ggtheme = ggplot2::theme_bw()

# df0 <- Fruit
# x <- 2
# y <- 6
# cov <- 8
# xlab0 = NULL
# ylab0 <- "Number of seeds"
# covlab <- "Sample blocks"
# letters0 <- c("a", "b", "c", "d", "e")
# letterscov <- rep(c("a", "b", "c"), 5)
# lettersheight0 = NULL
# letters0size = 6
# ggtheme = ggplot2::theme_bw()

#### FUNCTION ####
#### Figure barplot with means, standard error and letters ###
agbarplotinteraction <-
  function(df0, x, y, cov, ... , xlab0 = NULL, ylab0 = NULL, covlab = NULL,
           letters0 = NULL, letterscov = NULL, lettersheight0 = NULL,
           lettersheightcov = NULL, letters0size = 6, letterscovsize = 5,
           letters0col = "darkgrey", letterscovcol = "black",
           ggtheme = ggplot2::theme_bw(), width = NULL, dodge = NULL){
    #### Description ####
    ### INPUT
    ### df0, data frame
    ### x, a number for factor variable
    ### y, a number for numeric variable
    ### cov, a number for factor covariate variable
    ### xlab0, label for x variable
    ### ylab0, label for y variable
    ### covlab, label for covariate variable
    ### letters0, letters from anova or Kruskal-Wallis post hoc. If null not added
    ### letterscov, letters from post hoc analyses between covariate groups.
    ### lettersheight0, vertical position for letters0
    ### lettersheightcov, vertical position for letterscov
    ### letters0size, size of letters for x levels
    ### letterscovsize, size of letters for covariate into x levels
    ### letters0col, colour of letters for x levels
    ### letterscovcol, colour of letters for covariate into x levels,
    ### ggtheme, theme for ggplot
    #### OUTPUT
    ### A ggplot object with a barplot with covariate bars for each x variable.

    #### Require ####
    # agrequire("ggplot2")

    #### Check variables ####
    if (1 == 1) {
      ### df0 ####
      df0 <- as.data.frame(df0)
      if (!is.data.frame(df0)) stop("df0 needs to be a data frame in agbarplotinteraction()")

      ### x numeric or character ####
      if (is.character(x)) {
        # nx <- x
        if (!any(names(df0) %in% x)) {
          stop(paste("x is not a variable name of df0 in agbarplotinteraction().",
                     "It can be a variable name or number."))
        }
        if (any(names(df0) %in% x)) x <- which(names(df0) %in% x)

      }
      if (!is.integer(x)) x <- as.integer(x)
      if (!is.integer(x)) stop(paste("x is not a variable number of df0 in",
                                     "agbarplotinteraction(). It can be a variable name or",
                                     " number."))
      ### x as factor ####
      if (!is.factor(df0[[x]])) df0[[x]] <- as.factor(df0[[x]])
      if (!is.factor(df0[[x]])) {
        stop("In agbarplotinteraction(), x needs to point to a factor variable.")
      }

      ### y numeric or character ####
      if (is.character(y)) {
        # ny <- y
        if (!any(names(df0) %in% y)) {
          stop(paste("y is not a variable name of df0 in agbarplotinteraction().",
                     "It can be a variable name or number."))
        }
        if (any(names(df0) %in% y)) y <- which(names(df0) %in% y)

      }
      if (!is.integer(y)) y <- as.integer(y)
      if (!is.integer(y)) stop(paste("y is not a variable number of df0 in",
                                     "agbarplotinteraction(). It can be a variable name or",
                                     " number."))

      ### y as numeric ####
      if (!is.numeric(df0[[y]])) df0[[y]] <- as.numeric(df0[[y]])
      if (!is.numeric(df0[[y]])) {
        stop("In agbarplotinteraction(), y needs to point to a numeric variable.")
      }

      ### cov numeric or character ####
      if (is.character(cov)) {
        # ny <- cov
        if (!any(names(df0) %in% cov)) {
          stop(paste("cov is not a variable name of df0 in agbarplotinteraction().",
                     "It can be a variable name or number."))
        }
        if (any(names(df0) %in% cov)) cov <- which(names(df0) %in% cov)

      }
      if (!is.integer(cov)) cov <- as.integer(cov)
      if (!is.integer(cov)) stop(paste("cov is not a variable number of df0 in",
                                       "agbarplotinteraction(). It can be a variable name or",
                                       " number."))

      ### cov as factor ####
      if (!is.factor(df0[[cov]])) df0[[cov]] <- as.factor(df0[[cov]])
      if (!is.factor(df0[[cov]])) {
        stop("In agbarplotinteraction(), cov needs to point to a factor variable.")
      }


      ### letterscov ####
      ### should be same length than number of columns
    }

    #### Variables names for labels ####
    if (is.null(xlab0)) xlab0 <- names(df0)[x]
    if (is.null(ylab0)) ylab0 <- names(df0)[y]
    if (is.null(covlab)) covlab <- names(df0)[cov]

    #### Means and standard errors ####
    # df0[y] %>%
    #   group_by(df0[,x]) %>%
    #   summarise_each(funs(mean, sd))
    df1 <- as.data.frame(df0[c(y, cov, x)])
    df2 <- aggregate(df1[,1] ~ df1[,2] * df1[,3], df1,
                     function(x) c(mean = mean(x),
                                   sd = sd(x),
                                   #computation of the standard error of the mean
                                   se = sd(x)/sqrt(length(x))
                     ))
    df2 <- cbind(df2[1:2], df2[[3]])
    # str(df2)
    names(df2) <- c("covar", "varx", "Mean", "sd0", "se0")

    ### as.factors
    # df2$covar

    #### Plot ####
    BP <- ggplot(df2, aes(x = varx, y = Mean,
                          # color = covar,
                          fill = covar)) +
      # geom_bar(stat = "identity") +
      geom_bar(stat = "identity",  width = width,
               position = position_dodge(dodge)) +
      geom_errorbar(position = position_dodge(width = dodge), colour = "black",
                    aes(ymin = Mean - se0, ymax = Mean + se0, width = .1)) +

      ### Axis labels
      labs(x = xlab0, y = ylab0) +

      ### Legend remove
      guides(fill = guide_legend(title = covlab), color = FALSE)

    #### Add Theme ####
    BP <- BP + ggtheme

    #### Add letters0 to plot ####
    if (!is.null(letters0)) {                               # If there are letters
      #### If number of letters is not right ####
      if (!length(levels(df0[[x]])) == length(letters0)) {
        stop(paste("In agbarplotinteraction(),",
                   "letters0 must be same length than levels of x variable."))
      }

      #### If number of letters is right ####
      if (length(levels(df0[[x]])) == length(letters0)) {
        #### Calculate letters height if necessary ####
        if (!is.numeric(lettersheight0)) lettersheight0 <-
            max(df2$Mean) + max(df2$Mean) / 5
        #### Data frame for letters ####
        Dletters <- data.frame(varx = levels(df0[[x]]), yy = letters0, covar = NA)

        #### Add letters to plot ####
        BP <- BP + geom_text(data = Dletters,
                             aes(y = lettersheight0, x = varx, label = yy),
                             colour = letters0col, size = letters0size)
      }
    }

    #### Add letterscov to plot ####
    if (!is.null(letterscov)) {                               # If there are letters
      #### If number of letters is not right ####
      if (!length(df2[[1]]) == length(letterscov)) {
        stop(paste("In agbarplotinteraction(), letterscov must be same length than",
                   "interaction levels of x variable:", length(df2[[1]])))
      }

      #### If number of letters is right ####
      if (length(df2[[1]]) == length(letterscov)) {
        #### Calculate letters height if necessary ####
        if (!is.numeric(lettersheightcov)) lettersheightcov <-
            mean(na.exclude(df0[[y]])) / 4
        #### Data frame for letters ####
        # Dletters <- data.frame(varx = levels(df0[[x]]), yy = letters0, covar = NA)

        #### Add letters to plot ####
        BP <- BP + geom_text(aes(y = lettersheightcov, label = letterscov),
                             position = position_dodge(width = dodge),
                             colour = letterscovcol, size = letterscovsize)
      }
    }

    #### Return ####
    return(BP)
  }

