####### Function agcheckfactor check factor in df and number of levels  ########
### Created: 180621 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### V0.0.1: Working version.
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#### Roxigen ####
#' Check factor.
#'
#' Checks if the chosen variable(s) is a factor. If not, it is coerced to factor.
#'
#' Checks if the length of the factor levels is less than 'maxlevels'.
#'
#' @param df0 Data frame
#' @param v A number or string (or vector with numbers or names) referring
#'        to dependent variable(s) or target variables.
#' @param maxlevels The maximum number of levels allowed for factors.
#' @param ... Names needed for next parameters.
#' @param funame Name of the function. Needed to point error messages.
#'        Default to "agcheckbar".
#' @param vname Name of the variable. Needed to point error messages.
#'        Default to "v".
#' @param dfname Name of the data frame. Needed to point error messages.
#'        Default to "df0".
#'
#' @return
#' The data frame *df0* with the coerced factors.
#'
#' Either error or warning if something is not right.
#' Errors may have the name of
#' the main function (if included in *funame*), the name of the variable
#' (if included in *vname*) and the name of the data frame (if included in
#' *dfname*).
#'
#' @export
#'
#' @examples
#'
#' agcheckfactor(df0 = iris, v = 5)
#' A <- agcheckfactor(df0 = iris, v = c(1,5))
#' str(A)
#' agcheckfactor(df0 = iris, v = c(1,5), maxlevels = 9) # error
#' agcheckfactor(iris, 6) # error
#' agcheckfactor(iris, "Species")
#' str(agcheckfactor(iris, c("Species",  "Petal.Length", "Sepal.Width")))
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#### Init ####
# df0 <- iris
# v <- 5
# maxlevels = 9
# funame = "Function_name"
# vname = "Variable_name"
# dfname = "Dataframe_name"

#### Function ####
agcheckfactor <- function(df0, v, maxlevels = NULL, ..., funame = "Function_name",
                       vname = "Variable_name", dfname = "Dataframe_name") {
  #### check df0 ####
  df0 <- agcheckdf(df0, dfname = dfname, funame = funame)

  #### check vars ####
  v <- agcheckvar(df0, v, funame = funame, vname = vname, dfname = dfname)

  # #### Check factor ####
  i = v[1]
  for (i in v) {
      txt <- paste0(funame, '(): Variable "', names(df0)[i],
                    '" (', i, ') in "', dfname, '"')
    if (!is.factor(df0[[i]])) {
      warning(paste(txt, 'is not a factor. Trying to coerce.'))

      df0[[i]] <- as.factor(df0[[i]]) # coerce

      ### if still no factor
      if (!is.factor(df0[[i]])) stop(paste(txt, 'is not a factor.'))
     } # if i is not factor

    if (length(levels(df0[[i]])) < 1) stop(paste(txt, 'has less than 1 level.'))
    if (length(levels(df0[[i]])) == 1) stop(paste(txt, 'has only 1 level.'))

    #### maxlevels ####
    if (!is.null(maxlevels) && length(levels(df0[[i]])) > maxlevels)
      stop(paste0(txt, ' has more than the maximum allowed number of levels (',
                  maxlevels, '). Change "maxlevels" if desired.'))

  } # for each variable in v
  return(df0)
}
