####### Function agkruskalfigs One table histogram and densities ###############
### Created: 170926 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### v0.0.4 180512: Add roxygen2 documentation.
### v1.0.3 180503: Add captions
### v1.0.2 180418: Fixed some bugs
### V1.0.1 171202: Working version
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#### Roxigen ####
#' Kruskal Wallis test, post hoc and figures.
#'
#' Kruskal Wallis test for multiple comparison. Table and figures for
#' representing the results
#'
#' Post hoc test is using the criterium Fisher's least significant difference.
#' Alpha is 0.05 by default.
#'
#' The adjustment methods is Holm's method. There seems no reason to use the
#' unmodified Bonferroni correction because it is dominated by Holm's method,
#' which is also valid under arbitrary assumptions (Holm 1979).
#'
#' @references
#' Holm, S. (1979). A simple sequentially rejective multiple test procedure.
#'     Scandinavian Journal of Statistics 6, 65–70.
#'
#' @param df0 A data frame.
#' @param x String pointing to the x variable inside "df0"
#'   dataframe. Should be a factor variable.
#' @param y String pointing to the y variable inside "df0"
#'   dataframe. Should be a numeric variable.
#' @param xlab Changes the x axis label in barplot.
#' @param ylab Changes the y axis label in barplot.
#' @param lettersheight Changes the letters0 height in the barplot.
#'   Default = mean(df0[[y]]) / 4
#'
#' @return
#' A list with:
#'
#' - $tables$kw <-  Table with Principal statistics: mean, standard deviation,
#'       standard error, skewness, kurtosis, shapiro.test for normality,
#'       Kruskal Wallis.
#'
#' - $captables$kw <- Caption.
#'
#' - $plots$BarPlot <- Barplot.
#'
#' - $plots$Histogram <- Histogram.
#'
#' - $capplots$BarPlot <- Caption for barplot.
#'
#' - $capplots$Histogram <- Caption for histogram.
#'
#' - $plots$Densities <- Densities plot.
#'
#' - $capplots$Densities <- Caption for densities plot.
#'
#' - $statistics <- Statistics from Kruskal-Wallis analysis.
#'
#' - $parameters <- Parameters from Kruskal-Wallis analysis.
#'
#' @import agricolae
#'
#' @export
#'
#' @examples
#' A <- agkruskalfigs(df0 = iris, x = "Species", y = "Petal.Length",
#'              lettersheight = 1)
#' A$plots$BarPlot
#' A$plots$Densities
#' A$tables$kw
#'
#' @seealso
#' \url{http://www.biostathandbook.com/kruskalwallis.html}
#' for help on Kruskall Wallis analysis. Also
#' \code{\link[agricolae]{kruskal}},
#' \code{\link[ggpubr]{ggdensity}} and
#' \code{\link[plyr]{ddply}} to see the details of the functions used.
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#' @note
#' Future improvements:
#'
#' - The same without plyr
#'
#' - Add Levenne test to table (car::leveneTest()).
#'
#### Init ####
# df0 <- iris
# x <- 5
# y <- 1
# xlab0 = NULL
# ylab0 = c("mean", "density")
# lettersheight = NULL
# maxlevels = 9

#### FUNCTION ####
agkruskalfigs <- function(df0, x, y, xlab0 = NULL, ylab0 = c("mean", "density"),
                    lettersheight = mean(df0[[y]]) / 4, maxlevels = 9){
  #### Check ####
  if (TRUE) {
    #### Only one variable x and y ####
    if (length(x) != 1) stop("agkruskalfigs(): 'x' must be ONE variable.")
    if (length(y) != 1) stop("agkruskalfigs(): 'y' must be ONE variable.")

        #### df0 ####
    df0 <- na.exclude(data.frame(df0[x], df0[y]))
    df0 <- agcheckdf(df0, dfname = "df0", funame = "agkruskalfigs")

    #### alpha0 ####
    # if (!is.numeric(alpha0)) stop("agkruskalfigs(): 'alpha0' must be numeric.")
    # if (alpha0 <= 0 || alpha0 >= 1) stop(paste("agkruskalfigs(): 'alpha0' must be",
    #                                            "a number between 0 and 1."))
    #
    #### Variables assignations ####
    x <- agcheckvar(df0 = df0, v = 1, funame = "agkruskalfigs", vname = "x",
                    dfname = "df0")
    y <- agcheckvar(df0 = df0, v = 2, funame = "agkruskalfigs", vname = "y",
                    dfname = "df0")


    #### Factor variable x ####
    df0 <- agcheckfactor(df0, 1, maxlevels = maxlevels,
                         funame = "agkruskalfigs", dfname = "df0", vname = "x")

    #### Numeric variables y ####
    ### Coerce
    df0[[2]] <- as.numeric(df0[[2]])

    ### check
      if (!is.numeric(df0[[2]])) {
        nam <- names(df0[2])
        stop(paste0("agkruskalfigs(): 'y' variable must be numeric. '",
                    nam, "' is not numeric."))
      } # if not numeric
  } # All checks

  #### EXTRACT DATA FOR ANALYSIS ####
  xn <- names(df0)[1]
  yn <- names(df0)[2]
  if (is.null(xlab0)) xlab0 <- xn

  names(df0) <- c("x", "y")

  if (is.null(lettersheight)) lettersheight = mean(df0$y) / 4
  #### Init table of means A ####
  ll <- length(levels(df0$x))
  ta <- data.frame(x = factor(levels(df0$x), levels = levels(df0$x)),
                     N = numeric(ll),
                     Mean = numeric(ll),
                     sd = numeric(ll),
                     se = numeric(ll),
                     skew = numeric(ll),
                     kurtosis = numeric(ll),
                     Shapiro = numeric(ll)
  )
  names(ta)[1] <- xlab0

  #### Table of means ####
  df0$x <- as.factor(df0$x)
  # i = 1
  for (i in 1:ll) {
    s <- subset(df0, df0$x == levels(df0$x)[i])
    ta$N[i] <- length(s$y)
    ta$Mean[i] <- mean(s$y, na.rm = TRUE)
    ta$sd[i] <- sd(s$y)
    ta$se[i] <- sd(s$y)/sqrt(length(s$y))
    ta$skew[i] <- agricolae::skewness(s$y)
    ta$kurtosis[i] <- agricolae::kurtosis(s$y)
    ta$Shapiro[i] <- ifelse(is.nan(agricolae::skewness(s$y)), NA,
                            shapiro.test(s$y)$p.value)
  }

  #### KRUSKAL WALLIS ####
  KW <- with(df0, agricolae::kruskal(df0[2], df0[1], p.adj = "holm"))
  groups <- KW$groups
  names(groups)[2] <- "KW"  # Añadido tras actualizacion 170922 (groups ha eliminado columna de nombres tratamientos)
    groups$rownames <- rownames(groups)                 # Añadido tras actualizacion 170922
    names(groups)[names(groups) == "rownames"] <- xlab0     # Añadido tras actualizacion 170922

    ### Join tables
    # names(groups) <- c(xlab, "KW")                    # eliminado tras actualizacion 170922
    TabA <- plyr::join(ta, groups[c(2, 3)], by = xlab0)
    TabA <- TabA[c(1:3,9,4:8)]

    ### df0 for figures
    datosF <- TabA[c(1,3,4,6)]
    datosF[,xlab0] <- factor(datosF[,xlab0])
    datosF$Mean <- as.numeric(datosF$Mean)
    datosF$se <- as.numeric(datosF$se)

    #### Caption for table ####
    captable <- paste0("Kruskall-Wallis for the effect of ", xlab0, " on ", yn,
                      ". Chisq_p = ", format(KW$statistics$p.chisq, digits = 2),
                      ". Different letters on HSD mean significant ",
                      "differences for alpha = ", KW$parameters$alpha, ".")

    #### Figure barplot with means, standard error and letters ####
    BP <- ggplot(datosF, aes(x = datosF[,xlab0]
                            , y = Mean
                            , palette = "jco")
                ) +
        geom_bar(stat = "identity", aes(color = datosF[,xlab0]
                                        , fill = datosF[,xlab0]
                                        # , alpha = 0.5
                                        )
                ) +
        geom_errorbar(aes(ymin = Mean - se, ymax = Mean + se, width = .1)) +
        ### Label for Block pair
        geom_text(aes(y = lettersheight, label = datosF$KW),
                  colour = "black",
                  size = 6) +
        ### Axis labels
        labs(x = xlab0
            , y = ylab0[1]
            , color = xlab0, fill = xlab0
            ) +
        guides(fill = FALSE, color = FALSE) +
        ### Theme
        theme_classic()

    #### caption for barplot ####
    capbarplot <-
      paste0("Barplot for the effect of ", x, " on ", y,
             ". Different letters mean significant differences in ",
             "Kruskal-Wallis test by ranks.")

    ####  Histogram ####
    # df0
    HS <- ggplot(data = df0, aes(x = df0$y)) +
        geom_density(col = 2)  +
        geom_histogram(aes(y = ..density..),
                       breaks = seq(min(df0$y),max(df0$y),by = 1),
                       col = "brown",
                       fill = "brown",
                       alpha = .6) +
        ### Axis labels
        labs(x = y
            , y = ylab0[2]
            ) +
        ### Theme
        theme_classic()

    caphist <- paste0("Histogram for ", y, ".")

    #### OUTPUT ####
    ### Create list for output
    LS <- list()

    ### Include table into list
    LS$tables$kw <- TabA
    LS$captables$kw <- captable


    ### Include plots into list
    LS$plots$BarPlot <- BP
    LS$plots$Histogram <- HS

    LS$capplots$BarPlot <- capbarplot
    LS$capplots$Histogram <- caphist

    #### Densities plot ####
    LS$plots$Densities <-
      ggpubr::ggdensity(data = df0, x = 'y'
                        , add = "mean"
                        , rug = TRUE
                        , color = 'x', fill = 'x'
                        , alpha = 0.1
                        # , palette = "jco"
                        # , palette = "npg"
                        , palette = "lancet"
                        , xlab = ""
      )

    LS$capplots$Densities <-
      paste0("Densities for ", yn, " according to ", xlab0,
                             ". Dotted lines represent the mean value.")

    #### Statistics and parameters ####
    LS$statistics <- KW$statistics
    LS$parameters <- KW$parameters

    return(LS)
}
