#### Function_agviolin_violin_plot_and_caption #################################
### Created: 180416 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### v0.0.3 180512: Add roxygen2 documentation
### V0.0.2 180510: letters and xangle.
### V0.0.1: working
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#### Roxigen ####
#' Violin plot
#'
#' Violin plot with boxplot inside and letters
#'
#' @param x Factor variable.
#' @param y Numeric variable.
#' @param ... names needed for next parameters.
#' @param xlab0 Changes the x axis label in barplot.
#' @param ylab0 Changes the y axis label in barplot.
#' @param letters0 Post Hoc letters to include into the barplot.
#' @param lettersheight Changes the letters height in the barplot.
#' Default = max(Data[, y]) + ((max(Data[, y]) - min(Data[, y])) / 4)
#' @param ggtheme Changes theme for barplot. Default = ggplot2::theme_bw().
#' @param xangle Changes the angle of text for x axis label in barplot.
#' @param hjust Changes the horizontal justification of text for x axis label
#' in barplot.
#'
#' @return
#' #' A list with the violin plot and its caption:
#'
#' - $plots, the violin plot.
#'
#' - $capplots, the caption for the violin plot.
#'
#' @import ggplot2
#'
#' @export
#'
#' @examples
#' agviolin(x = iris[5], y = iris[1])
#'
#' @details
#'
#' Requires ggplot2.
#'
#' @seealso
#' \code{\link{agautoanalyses::agrequire}},
#' \code{\link{ggplot2::ggplot}}
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#'
#### FUNCTION ####
agviolin <- function(x, y, ..., xlab0 = NULL, ylab0 = NULL,
                     letters0 = NULL,
                     lettersheight = max(y) + ((max(y) - min(y)) / 4),
                     ggtheme = ggplot2::theme_bw(),
                     xangle = NULL, hjust = NULL) {
  #### require ####
  # agrequire("ggplot2")

  #### Data frame ####
  n <- c(names(x), names(y))
  d1 <- data.frame(nx = x[[1]], ny = y[[1]])
  # names(d1) <- n
  d1 <- na.exclude(d1)

  #### Variables names for labels ####
  if (is.null(xlab0)) xlab0 <- names(x)
  if (is.null(ylab0)) ylab0 <- names(y)

  #### plot ####
  plotv <- ggplot(d1, aes(x = nx, y = ny)) +
    geom_violin(trim = FALSE) +
    labs(x = n[1], y = n[2]) +
    geom_boxplot(width = 0.1) +
    labs(x = xlab0, y = ylab0) +
    ggtheme

  #### Add letters to plot ####
  D <- data.frame(x = levels(as.factor(x[[1]])),
                  y = rep(lettersheight))
  if (!is.null(letters0)) {
    plotv <- plotv + geom_text(data = D, aes(x = x, y = y, label = letters0),
                               colour = "black",
                               size = 6)
  }

  #### rotate xlab if necessary ####
  plotv <- plotv +
    theme(axis.text.x = element_text(angle = xangle, hjust = hjust))

  #### return ####
  out <- list()
  out$plots <- plotv
  out$capplots <- paste0("Violin plot of ", n[2], " for each ", n[1], ".")

  return(out)
}


