####### Function agxlsx2csv imports xlsx cleans and export csv #################
### Created: 181025 Author: Alfonso Garmendia  algarsal at upvnet dot upv dot es

### v0.0.1: working.
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#### Roxigen ####
#' Converts xlsx to csv
#'
#' Imports an xlsx file, cleans it from commas and accents and export it to a
#' csv file.
#'
#' @references
#'
#' @param xls String pointing to the xlsx file.
#' @param csv String pointing to the direction for the csv file.
#'     Default = the xls file with csv extension.
#' @param sheet String or integer pointing to the spreadsheet into xls.
#'     Default = 1.
#' @param skip Minimum number of rows to skip before reading anything,
#'     be it column names or data. Leading empty rows are automatically
#'     skipped, so this is a lower bound. Ignored if range is given.
#' @param n_max Maximum number of data rows to read.
#'     Trailing empty rows are automatically skipped,
#'     so this is an upper bound on the number of rows in the returned tibble.
#' @param clean Logic. If true, cleans the data frame.
#'
#' @return
#'     Returns the dataframe with the data
#'
#' @import readxl
#'     tools
#'
#' @export
#'
#' @examples
#' agxlsx2csv(xls = existing-file.xlsx)
#'
#' @seealso
#' \code{\link[readxl]{read_excel}},
#' \code{\link[tools]{file_path_sans_ext}},
#' \code{\link[utils]{write.csv}}
#'
#' @author Alfonso Garmendia (Universitat Politècnica de València)
#'
#' @note
#'
#### Init ####
# xls <- "data/xls_original/Resultados temperatura polen rosáceas 20180924.xlsx"
# csv = NULL
# sheet = NULL
# skip = 0
# n_max = Inf
# clean = TRUE

#### FUNCTION ####
agxlsx2csv <- function(xls, csv = NULL, sheet = NULL, skip = 0, n_max = Inf,
                       clean = TRUE){
  #### Check classes ####
  if (!"character" %in% class(xls))
    stop("agxlsx2csv(): 'xls' must be a character string.")

  if (!is.null(csv) & !"character" %in% class(csv))
    stop("agxlsx2csv(): 'csv' must be a character string.")

  if (!is.null(sheet) & !"character" %in% class(sheet))
    stop("agxlsx2csv(): 'sheet' must be a character string.")

  as.integer(skip)
  if (!"numeric" %in% class(skip))
    stop("agxlsx2csv(): 'skip' must be an integer number.")

  if (!"numeric" %in% class(n_max))
    stop("agxlsx2csv(): 'n_max' must be an integer number.")

  if (!"logical" %in% class(clean))
    stop("agxlsx2csv(): 'clean' must be logical (TRUE or FALSE).")

  #### EXTRACT DATA ####
  ### Read excel files
  # dir("data/xls_original", full.names = TRUE)
  d <- read_excel(path = xls, sheet = sheet, skip = skip, n_max = n_max)

  #### CLEAN ####
  if (clean) {
    #### Elliminate commas ####
    # i <- names(d)[1]
    for (i in names(d)) {
      if (is.character(d[[i]])) d[[i]] <- gsub(",", "", d[[i]])
      if (is.numeric(d[[i]])) d[[i]] <- as.numeric(gsub(",", "", d[[i]]))
    }

    #### Elliminate accents from names ####
    ac0 <- c(LETTERS, "á", "é", "í", "ó", "ú", "à", "è", "ò", "ñ", "-", " ",
             "Á", "É", "Í", "Ó", "Ú", "À", "È", "Ò")
    ac1 <- c(letters, "a", "e", "i", "o", "u", "a", "e", "o", "n", ".", "_",
             "a", "e", "i", "o", "u", "a", "e", "o")

    # i <- 1
    for (i in 1:length(ac0)) {
      names(d) <- gsub(ac0[i], ac1[i], names(d))
    }


  }
  #### Export CSV ####
  if (is.null(csv)) {
    # library(tools)
    # dn <- dirname(xls)
    csv <- paste0(file_path_sans_ext(xls), ".csv")
  }

  write.csv(x = d, file = csv, fileEncoding = "UTF-8", quote = FALSE, na = "",
            row.names = FALSE)

  return(d)

}
